@@ -34,7 +34,7 @@ class ThesaurusTermsSearchView(MethodPublisher):
     """Thesaurus terms search view"""
 
     def findTerms(self, query, thesaurus_name='', extract_name='',
-                  autoexpand='on_miss', glob='end', limit=20):
+                  autoexpand='on_miss', glob='end', limit=50):
         allow_archives = True
         if IThesaurus.providedBy(self.context):
             thesaurus = self.context
@@ -43,13 +43,14 @@ class ThesaurusTermsSearchView(MethodPublisher):
             if thesaurus is None:
                 return []
             allow_archives = False
-        return [ { 'value': term.caption,
-                   'caption': term.caption }
-                 for term in unique(thesaurus.findTerms(query, extract_name, autoexpand, glob, limit), idfun=lambda x: x.caption)
-                          if allow_archives or (term.status != STATUS_ARCHIVED) ]
+        return [{'value': term.caption,
+                 'caption': term.caption}
+                for term in unique(thesaurus.findTerms(query, extract_name, autoexpand, glob, limit, exact=True),
+                                   idfun=lambda x: x.caption)
+                         if allow_archives or (term.status != STATUS_ARCHIVED)]
 
     def findTermsWithLabel(self, query, thesaurus_name='', extract_name='',
-                           autoexpand='on_miss', glob='end', limit=20):
+                           autoexpand='on_miss', glob='end', limit=50):
         allow_archives = True
         if IThesaurus.providedBy(self.context):
             thesaurus = self.context
@@ -58,7 +59,8 @@ class ThesaurusTermsSearchView(MethodPublisher):
             if thesaurus is None:
                 return []
             allow_archives = False
-        return [ { 'value': term.label,
-                   'caption': term.label }
-                 for term in unique(thesaurus.findTerms(query, extract_name, autoexpand, glob, limit), idfun=lambda x: x.label)
-                          if allow_archives or (term.status != STATUS_ARCHIVED) ]
+        return [{'value': term.label,
+                 'caption': term.label}
+                for term in unique(thesaurus.findTerms(query, extract_name, autoexpand, glob, limit, exact=True),
+                                   idfun=lambda x: x.label)
+                         if allow_archives or (term.status != STATUS_ARCHIVED)]
