@@ -27,10 +27,10 @@ from zope.schema.interfaces import IVocabularyFactory
 from zope.size.interfaces import ISized
 
 # import local interfaces
-from ztfy.blog.browser.interfaces.container import ITitleColumn, IActionsColumn, \
-                                                   IContainerTableViewTitleCell, IContainerTableViewActionsCell
 from ztfy.security.interfaces import ISecurityManager
-from ztfy.skin.interfaces import IContainedDefaultView, IContainerBaseView, IDefaultView
+from ztfy.skin.interfaces import IContainedDefaultView, IDefaultView
+from ztfy.skin.interfaces.container import IContainerBaseView, ITitleColumn, IActionsColumn, \
+                                           IContainerTableViewTitleCell, IContainerTableViewActionsCell
 from ztfy.skin.layer import IZTFYBrowserLayer, IZTFYBackLayer
 from ztfy.thesaurus.browser.interfaces import IThesaurusView
 from ztfy.thesaurus.browser.interfaces.thesaurus import IThesaurusAddFormMenuTarget, \
