@@ -53,7 +53,7 @@ class ThesaurusTermFieldIndex(ValueIndex):
             return None
 
         intids = getUtility(IIntIds)
-        value = '%s-kw:%d' % (self.thesaurus_name + '::' if self.thesaurus_name else '', intids.getId(value))
+        value = '%s-kw:%d' % (self.thesaurus_name + '::' if self.thesaurus_name else '', intids.register(value))
         return ValueIndexBase.index_doc(self, docid, value)
 
 
@@ -78,5 +78,6 @@ class ThesaurusTermsListFieldIndex(SetIndex):
             return None
 
         intids = getUtility(IIntIds)
-        value = [ '%s-kw:%d' % (self.thesaurus_name + '::' if self.thesaurus_name else '', intids.getId(term)) for term in value ]
+        value = [ '%s-kw:%d' % (self.thesaurus_name + '::' if self.thesaurus_name else '', intids.register(term))
+                  for term in value ]
         return SetIndexBase.index_doc(self, docid, value)
