@@ -37,6 +37,7 @@ from zope.traversing.api import getName
 
 # import local packages
 from ztfy.utils.timezone import tztime
+from ztfy.utils.unicode import translateString
 
 
 REVERSE_LINK_ATTRIBUTES = { 'generic': 'specifics',
@@ -89,6 +90,10 @@ class ThesaurusTerm(Persistent, Contained):
         else:
             return isinstance(removeSecurityProxy(other), ThesaurusTerm) and (self.label == other.label)
 
+    @property
+    def base_label(self):
+        return translateString(self.label, escapeSlashes=True, forceLower=True, spaces=' ')
+
     @property
     def caption(self):
         if self._usage:
