@@ -20,6 +20,8 @@ import BTrees.Length
 
 # import Zope3 interfaces
 from hurry.query.interfaces import IQuery
+from transaction.interfaces import ITransactionManager
+from zc.catalog.interfaces import IValueIndex
 from zope.annotation.interfaces import IAnnotations
 from zope.catalog.interfaces import ICatalog
 from zope.container.interfaces import IObjectAddedEvent, IObjectRemovedEvent
@@ -37,6 +39,8 @@ from ztfy.thesaurus.interfaces.tree import ITree
 
 # import Zope3 packages
 from hurry.query import And
+from hurry.query.value import Eq
+from zc.catalog.catalogindex import ValueIndex
 from zope.catalog.catalog import Catalog
 from zope.component import adapter, adapts, getUtility, queryUtility, getSiteManager
 from zope.container.contained import Contained
@@ -52,6 +56,7 @@ from ztfy.utils.catalog import indexObject, unindexObject
 from ztfy.utils.catalog.index import TextIndexNG, Text as BaseTextSearch
 from ztfy.security.property import RolePrincipalsProperty
 from ztfy.utils.traversing import getParent
+from ztfy.utils.unicode import translateString
 
 
 class ThesaurusIndexSearch(BaseTextSearch):
@@ -67,6 +72,19 @@ class ThesaurusIndexSearch(BaseTextSearch):
         return index
 
 
+class ThesaurusValueSearch(Eq):
+    """Custom value search index using catalog instance instead of name"""
+
+    def getIndex(self, context=None):
+        if ICatalog.providedBy(self.catalog_name):
+            catalog = self.catalog_name
+        else:
+            catalog = getUtility(ICatalog, self.catalog_name, context)
+        index = catalog[self.index_name]
+        assert IValueIndex.providedBy(index)
+        return index
+
+
 class ThesaurusTermsContainer(Folder):
     """Thesaurus terms container"""
 
@@ -184,6 +202,7 @@ class Thesaurus(Persistent, Contained):
     def initCatalog(self):
         catalog = self.catalog = Catalog()
         locate(catalog, self, '++catalog++')
+        # init fulltext search catalog
         index = TextIndexNG('label', IThesaurusTerm,
                             field_callable=False,
                             languages=self.language,
@@ -192,18 +211,25 @@ class Thesaurus(Persistent, Contained):
                             use_stopwords=True,
                             use_normalizer=True,
                             ranking=True)
-        locate(index, catalog, 'label"')
+        locate(index, catalog, 'label')
         catalog['label'] = index
+        # init basic label catalog
+        index = ValueIndex('base_label', IThesaurusTerm, field_callable=False)
+        locate(index, catalog, 'base_label')
+        catalog['base_label'] = index
+        # index thesaurus terms
         locate(self.terms, self, '++terms++')
-        for term in self.terms.itervalues():
+        for index, term in enumerate(self.terms.itervalues()):
             indexObject(term, catalog)
+            if not index % 100:
+                ITransactionManager(catalog).savepoint()
 
     def delete(self):
         manager = getSiteManager(self)
         manager.unregisterUtility(self, IThesaurus, self.name)
         default = manager['default']
         name = 'Thesaurus::' + self.name
-        # reparent thesaurus to site management folder
+        # re-parent thesaurus to site management folder
         locate(self, default, name)
         del default[name]
 
@@ -226,7 +252,7 @@ class Thesaurus(Persistent, Contained):
         if thesaurus is not None:
             # define or merge items from given thesaurus
             terms = self.terms
-            for key, term in thesaurus.terms.iteritems():
+            for index, (key, term) in enumerate(thesaurus.terms.iteritems()):
                 # check for term conflict
                 if configuration.conflict_suffix:
                     suffixed_key = key + ' ' + configuration.conflict_suffix
@@ -238,11 +264,13 @@ class Thesaurus(Persistent, Contained):
                     terms[key].merge(term, configuration)
                 elif not IThesaurusLoaderTerm.providedBy(term):
                     terms[key] = term
+                if not index % 100:
+                    ITransactionManager(self).savepoint()
         self.resetTermsParent()
         self.resetTopTerms()
 
     def resetTermsParent(self):
-        for term in self.terms.itervalues():
+        for index, term in enumerate(self.terms.itervalues()):
             # reset generic/specifics attributes
             generic = term.generic
             if (generic is not None) and (term not in generic.specifics):
@@ -252,13 +280,20 @@ class Thesaurus(Persistent, Contained):
             while parent.generic is not None:
                 parent = parent.generic
             term.parent = parent
+            if not index % 100:
+                ITransactionManager(self).savepoint()
 
     def resetTopTerms(self):
         self.top_terms = [ term for term in self.terms.itervalues()
                            if (not term.generic) and (not term.usage) ]
 
-    def findTerms(self, query=None, extract=None, autoexpand='on_miss', glob='end', limit=None):
-        params = []
+    def findTerms(self, query=None, extract=None, autoexpand='on_miss', glob='end', limit=None, exact=False):
+        query_util = getUtility(IQuery)
+        terms = []
+        if exact:
+            search = ThesaurusValueSearch((self.catalog, 'base_label'),
+                                          translateString(query, escapeSlashes=True, forceLower=True, spaces=' '))
+            terms = list(query_util.searchResults(search))
         start = ''
         end = ''
         if '*' not in query:
@@ -266,14 +301,15 @@ class Thesaurus(Persistent, Contained):
                 start = u'*'
             if glob in ('end', 'both'):
                 end = u'*'
-        params.append(ThesaurusIndexSearch((self.catalog, 'label'), { 'query': u' '.join((u'%s%s%s' % (start, m, end) for m in query.split()
-                                                                                                                      if len(m) > 2)),
-                                                                      'autoexpand': autoexpand,
-                                                                      'ranking': True }))
-        terms = getUtility(IQuery).searchResults(And(*params), limit=limit)
+        search = ThesaurusIndexSearch((self.catalog, 'label'), {'query': u' '.join((u'%s%s%s' % (start, m, end)
+                                                                                    for m in query.split()
+                                                                                          if len(m) > 2)),
+                                                                'autoexpand': autoexpand,
+                                                                'ranking': False})
+        terms += sorted(query_util.searchResults(search, limit=limit), key=lambda x: x.label)
         if extract:
-            terms = [ term for term in terms
-                                    if extract in term.extracts ]
+            terms = [term for term in terms
+                                   if extract in term.extracts]
         return terms
 
 
